@@ -77,8 +77,6 @@ static char *get_reply(SPDConnection * connection);
 static int get_err_code(char *reply);
 static char *get_param_str(char *reply, int num, int *err);
 static int get_param_int(char *reply, int num, int *err);
-static void *xmalloc(size_t bytes);
-static void xfree(void *ptr);
 static int ret_ok(char *reply);
 static void SPD_DBG(char *format, ...);
 static void *spd_events_handler(void *);
@@ -161,17 +159,30 @@ static char *_get_default_unix_socket_name(void)
 	socket_filename = g_string_new("");
 	g_string_printf(socket_filename, "%s/speech-dispatcher/speechd.sock",
 			rundir);
-	// Do not regurn glib string, but glibc string...
+	// Do not return glib string, but glibc string...
 	h = strdup(socket_filename->str);
 	g_string_free(socket_filename, 1);
 	return h;
 }
 
+void SPDConnectionAddress__free(SPDConnectionAddress *address)
+{
+	if (!address)
+		return;
+	free(address->unix_socket_name);
+	free(address->inet_socket_host);
+	free(address->dbus_bus);
+	free(address);
+}
+
 SPDConnectionAddress *spd_get_default_address(char **error)
 {
 	const gchar *env_address = g_getenv("SPEECHD_ADDRESS");
 	gchar **pa;		/* parsed address */
 	SPDConnectionAddress *address = malloc(sizeof(SPDConnectionAddress));
+	address->unix_socket_name = NULL;
+	address->inet_socket_host = NULL;
+	address->dbus_bus = NULL;
 
 	if (env_address == NULL) {	// Default method = unix sockets
 		address->method = SPD_METHOD_UNIX_SOCKET;
@@ -205,7 +216,7 @@ SPDConnectionAddress *spd_get_default_address(char **error)
 			*error =
 			    strdup
 			    ("Unknown or unsupported communication method");
-			free(address);
+			SPDConnectionAddress__free(address);
 			address = NULL;
 		}
 		g_strfreev(pa);
@@ -247,7 +258,7 @@ SPDConnection *spd_open(const char *client_name, const char *connection_name,
 		_init_debug();
 		assert(error);
 		SPD_DBG("Could not connect to Speech Dispatcher: %s", error);
-		xfree(error);
+		free(error);
 	}
 	return conn;
 }
@@ -275,7 +286,7 @@ static char *resolve_host(char *host_name_or_ip, int *is_localhost,
 		*error =
 		    g_strdup_printf("Can't resolve address %d due to error %s:",
 				    err, gai_strerror(err));
-		xfree(resolve_buffer);
+		free(resolve_buffer);
 		return NULL;
 	}
 	/* Take the first address returned as we are only interested in host ip */
@@ -289,7 +300,7 @@ static char *resolve_host(char *host_name_or_ip, int *is_localhost,
 		    ("Could not convert address, due to the following error: %s",
 		     strerror(errno));
 		freeaddrinfo(addr_result);
-		xfree(resolve_buffer);
+		free(resolve_buffer);
 		return NULL;
 	}
 
@@ -297,7 +308,7 @@ static char *resolve_host(char *host_name_or_ip, int *is_localhost,
 		*is_localhost = 1;
 		/* In case of local addresses, use 127.0.0.1 which is guaranteed
 		   to be local and the server listens on it */
-		xfree(resolve_buffer);
+		free(resolve_buffer);
 		ip = strdup("127.0.0.1");
 	} else {
 		*is_localhost = 0;
@@ -371,10 +382,11 @@ spawn_server(SPDConnectionAddress * address, int is_localhost,
 
 SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 			 const char *user_name, SPDConnectionMode mode,
-			 SPDConnectionAddress * address, int autospawn,
+			 SPDConnectionAddress *address, int autospawn,
 			 char **error_result)
 {
 	SPDConnection *connection;
+	SPDConnectionAddress *defaultAddress = NULL;
 	char *set_client_name;
 	char *conn_name;
 	char *usr_name;
@@ -413,7 +425,8 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 
 	if (address == NULL) {
 		char *err = NULL;
-		address = spd_get_default_address(&err);
+		defaultAddress = spd_get_default_address(&err);
+		address = defaultAddress;
 		if (!address) {
 			assert(err);
 			*error_result = err;
@@ -423,7 +436,7 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	}
 
 	/* Connect to server using the selected method */
-	connection = xmalloc(sizeof(SPDConnection));
+	connection = malloc(sizeof(SPDConnection));
 	if (address->method == SPD_METHOD_INET_SOCKET) {
 		host_ip =
 		    resolve_host(address->inet_socket_host, &is_localhost,
@@ -431,6 +444,7 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 		if (host_ip == NULL) {
 			*error_result = strdup(resolve_error);
 			g_free(resolve_error);
+			SPDConnectionAddress__free(defaultAddress);
 			return NULL;
 		}
 		address_inet.sin_addr.s_addr = inet_addr(host_ip);
@@ -484,8 +498,10 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 				assert(0);
 			SPD_DBG(*error_result);
 			close(connection->socket);
+			SPDConnectionAddress__free(defaultAddress);
 			return NULL;
 		}
+		g_free(spawn_report);
 	}
 
 	if (address->method == SPD_METHOD_INET_SOCKET)
@@ -510,18 +526,18 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	if (ret)
 		SPD_FATAL("Can't set buffering, setvbuf failed.");
 
-	connection->ssip_mutex = xmalloc(sizeof(pthread_mutex_t));
+	connection->ssip_mutex = malloc(sizeof(pthread_mutex_t));
 	pthread_mutex_init(connection->ssip_mutex, NULL);
 
 	if (mode == SPD_MODE_THREADED) {
 		SPD_DBG
 		    ("Initializing threads, condition variables and mutexes...");
-		connection->events_thread = xmalloc(sizeof(pthread_t));
-		connection->cond_reply_ready = xmalloc(sizeof(pthread_cond_t));
+		connection->events_thread = malloc(sizeof(pthread_t));
+		connection->cond_reply_ready = malloc(sizeof(pthread_cond_t));
 		connection->mutex_reply_ready =
-		    xmalloc(sizeof(pthread_mutex_t));
-		connection->cond_reply_ack = xmalloc(sizeof(pthread_cond_t));
-		connection->mutex_reply_ack = xmalloc(sizeof(pthread_mutex_t));
+		    malloc(sizeof(pthread_mutex_t));
+		connection->cond_reply_ack = malloc(sizeof(pthread_cond_t));
+		connection->mutex_reply_ack = malloc(sizeof(pthread_mutex_t));
 		pthread_cond_init(connection->cond_reply_ready, NULL);
 		pthread_mutex_init(connection->mutex_reply_ready, NULL);
 		pthread_cond_init(connection->cond_reply_ack, NULL);
@@ -532,6 +548,7 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 		if (ret != 0) {
 			*error_result = strdup("Thread initialization failed");
 			SPD_DBG(*error_result);
+			SPDConnectionAddress__free(defaultAddress);
 			return NULL;
 		}
 	}
@@ -541,9 +558,10 @@ SPDConnection *spd_open2(const char *client_name, const char *connection_name,
 	    g_strdup_printf("SET SELF CLIENT_NAME \"%s:%s:%s\"", usr_name,
 			    client_name, conn_name);
 	ret = spd_execute_command_wo_mutex(connection, set_client_name);
-	xfree(usr_name);
-	xfree(conn_name);
-	xfree(set_client_name);
+	free(usr_name);
+	free(conn_name);
+	free(set_client_name);
+	SPDConnectionAddress__free(defaultAddress);
 	return connection;
 }
 
@@ -567,6 +585,10 @@ void spd_close(SPDConnection * connection)
 		pthread_cond_destroy(connection->cond_reply_ack);
 		pthread_join(*connection->events_thread, NULL);
 		connection->mode = SPD_MODE_SINGLE;
+		free(connection->mutex_reply_ready);
+		free(connection->mutex_reply_ack);
+		free(connection->cond_reply_ready);
+		free(connection->cond_reply_ack);
 	}
 
 	/* close the socket */
@@ -575,7 +597,8 @@ void spd_close(SPDConnection * connection)
 	pthread_mutex_unlock(connection->ssip_mutex);
 
 	pthread_mutex_destroy(connection->ssip_mutex);
-	xfree(connection);
+	free(connection->ssip_mutex);
+	free(connection);
 }
 
 /* Helper functions for spd_say. */
@@ -639,8 +662,8 @@ static inline int spd_say_sending(SPDConnection * connection, const char *text)
 		}
 	}
 
-	xfree(reply);
-	xfree(pret);
+	free(reply);
+	free(pret);
 	return msg_id;
 }
 
@@ -660,7 +683,7 @@ int spd_say(SPDConnection * connection, SPDPriority priority, const char *text)
 		if (!prepare_failed)
 			msg_id = spd_say_sending(connection, escaped_text);
 
-		xfree(escaped_text);
+		free(escaped_text);
 		pthread_mutex_unlock(connection->ssip_mutex);
 	} else {
 		SPD_DBG("spd_say called with a NULL argument for <text>");
@@ -689,7 +712,7 @@ spd_sayf(SPDConnection * connection, SPDPriority priority, const char *format,
 
 	/* Send the buffer to Speech Dispatcher */
 	ret = spd_say(connection, priority, buf);
-	xfree(buf);
+	free(buf);
 
 	return ret;
 }
@@ -783,7 +806,7 @@ spd_key(SPDConnection * connection, SPDPriority priority, const char *key_name)
 
 	command_key = g_strdup_printf("KEY %s", key_name);
 	ret = spd_execute_command_wo_mutex(connection, command_key);
-	xfree(command_key);
+	free(command_key);
 	if (ret)
 		RET(-1);
 
@@ -866,7 +889,7 @@ spd_sound_icon(SPDConnection * connection, SPDPriority priority,
 
 	command = g_strdup_printf("SOUND_ICON %s", icon_name);
 	ret = spd_execute_command_wo_mutex(connection, command);
-	xfree(command);
+	free(command);
 	if (ret)
 		RET(-1);
 
@@ -1020,7 +1043,7 @@ spd_w_set_voice_type(SPDConnection * connection, SPDVoiceType type,
         command = g_strdup_printf("SET %s " #param " %s", \
                               who, str); \
         ret = spd_execute_command(connection, command); \
-        xfree(command); \
+        free(command); \
         return ret; \
     } \
     int \
@@ -1201,13 +1224,14 @@ SPDVoice **spd_list_synthesis_voices(SPDConnection * connection)
 char **spd_execute_command_with_list_reply(SPDConnection * connection,
 					   char *command)
 {
-	char *reply = NULL, *line;
+	char *reply = NULL;
+	char *line;
 	int err;
 	int max_items = 50;
 	char **result;
-	int i, ret;
+	int i;
 
-	ret = spd_execute_command_with_reply(connection, command, &reply);
+	spd_execute_command_with_reply(connection, command, &reply);
 	if (!ret_ok(reply)) {
 		if (reply != NULL)
 			free(reply);
@@ -1220,7 +1244,7 @@ char **spd_execute_command_with_list_reply(SPDConnection * connection,
 		line = get_param_str(reply, i + 1, &err);
 		if ((err) || (line == NULL))
 			break;
-		result[i] = strdup(line);
+		result[i] = line;
 		if (i >= max_items - 2) {
 			max_items *= 2;
 			result = realloc(result, max_items * sizeof(char *));
@@ -1281,7 +1305,7 @@ int spd_execute_command(SPDConnection * connection, char *command)
 	if (ret) {
 		SPD_DBG("Can't execute command in spd_execute_command");
 	}
-	xfree(reply);
+	free(reply);
 
 	pthread_mutex_unlock(connection->ssip_mutex);
 
@@ -1299,7 +1323,7 @@ int spd_execute_command_wo_mutex(SPDConnection * connection, char *command)
 		SPD_DBG
 		    ("Can't execute command in spd_execute_command_wo_mutex");
 
-	xfree(reply);
+	free(reply);
 
 	return ret;
 }
@@ -1314,7 +1338,7 @@ spd_execute_command_with_reply(SPDConnection * connection, char *command,
 
 	buf = g_strdup_printf("%s\r\n", command);
 	*reply = spd_send_data_wo_mutex(connection, buf, SPD_WAIT_REPLY);
-	xfree(buf);
+	free(buf);
 	buf = NULL;
 	if (*reply == NULL) {
 		SPD_DBG
@@ -1395,7 +1419,7 @@ char *spd_send_data_wo_mutex(SPDConnection * connection, const char *message,
 				    ("Error: Can't read reply, broken socket in spd_send_data.");
 				return NULL;
 			}
-			xfree(connection->reply);
+			free(connection->reply);
 			bytes = strlen(reply);
 			if (bytes == 0) {
 				SPD_DBG("Error: Empty reply, broken socket.");
@@ -1485,7 +1509,7 @@ static char *get_reply(SPDConnection * connection)
 		/* terminate if we reached the last line (without '-' after numcode) */
 	} while (!errors && !((strlen(line) < 4) || (line[3] == ' ')));
 
-	xfree(line);		/* getline allocates with malloc. */
+	free(line);		/* getline allocates with malloc. */
 
 	if (errors) {
 		/* Free the GString and its character data, and return NULL. */
@@ -1534,6 +1558,7 @@ static void *spd_events_handler(void *conn)
 				SPD_DBG
 				    ("Bad reply from Speech Dispatcher: %s (code %d)",
 				     reply, err);
+				free(reply);
 				break;
 			}
 			client_id = get_param_int(reply, 2, &err);
@@ -1541,6 +1566,7 @@ static void *spd_events_handler(void *conn)
 				SPD_DBG
 				    ("Bad reply from Speech Dispatcher: %s (code %d)",
 				     reply, err);
+				free(reply);
 				break;
 			}
 			/*  Decide if we want to call a callback */
@@ -1569,21 +1595,23 @@ static void *spd_events_handler(void *conn)
 					SPD_DBG
 					    ("Broken reply from Speech Dispatcher: %s",
 					     reply);
+					free(reply);
 					break;
 				}
 				/* Call the callback */
 				connection->callback_im(msg_id, client_id,
 							SPD_EVENT_INDEX_MARK,
 							im);
-				xfree(im);
+				free(im);
 			}
+			free(reply);
 
 		} else {
 			/* This is a protocol reply */
 			pthread_mutex_lock(connection->mutex_reply_ready);
 			/* Prepare the reply to the reply buffer in connection */
 			if (reply != NULL) {
-				connection->reply = strdup(reply);
+				connection->reply = reply;
 			} else {
 				SPD_DBG("Connection reply is NULL");
 				connection->reply = NULL;
@@ -1598,7 +1626,6 @@ static void *spd_events_handler(void *conn)
 			pthread_cond_wait(connection->cond_reply_ack,
 					  connection->mutex_reply_ack);
 			pthread_mutex_unlock(connection->mutex_reply_ack);
-			xfree(reply);
 			/* Continue */
 		}
 	}
@@ -1702,7 +1729,7 @@ static int get_param_int(char *reply, int num, int *err)
 		*err = -3;
 		return 0;
 	}
-	xfree(rep_str);
+	free(rep_str);
 
 	return ret;
 }
@@ -1747,25 +1774,6 @@ static int isanum(char *str)
 	return 1;
 }
 
-static void *xmalloc(size_t bytes)
-{
-	void *mem;
-
-	mem = malloc(bytes);
-	if (mem == NULL) {
-		SPD_FATAL("Not enough memmory!");
-		exit(1);
-	}
-
-	return mem;
-}
-
-static void xfree(void *ptr)
-{
-	if (ptr != NULL)
-		free(ptr);
-}
-
 /*
  * escape_dot: Replace . with .. at the start of lines.
  * @text: text to escape
