@@ -301,7 +301,7 @@ OutputModule *load_output_module(char *mod_name, char *mod_prog,
 	}
 
 	if (s == '2')
-		MSG(2, "Module %s started sucessfully with message: %s",
+		MSG(2, "Module %s started successfully with message: %s",
 		    module->name, reply->str);
 
 	g_string_free(reply, 1);
@@ -538,3 +538,17 @@ void module_load_requested_modules(void)
 		    g_list_delete_link(requested_modules, requested_modules);
 	}
 }
+
+/*
+ * module_number_of_requested_modules: return the current number of requested modules.
+ * Returns: number of modules.
+ * Parameters: none.
+ */
+guint module_number_of_requested_modules(void)
+{
+	if (requested_modules == NULL) {
+		return 0;
+	}
+	return g_list_length(requested_modules);
+}
+
