@@ -23,24 +23,6 @@
 
 #include <getopt.h>
 
-static struct option spd_long_options[] = {
-    {"run-daemon", 0, 0, 'd'},
-    {"run-single", 0, 0, 's'},
-    {"spawn", 0, 0, 'a'},
-    {"log-level", 1, 0, 'l'},
-    {"communication-method", 1, 0, 'c'},
-    {"socket-path", 1, 0, 'S'},
-    {"port", 1, 0, 'p'},
-    {"pid-file", 1, 0, 'P'},
-    {"config-file", 1, 0, 'C'},
-    {"version", 0, 0, 'v'},
-    {"debug", 0, 0, 'D'},
-    {"help", 0, 0, 'h'},
-    {0, 0, 0, 0}
-};
-
-static char* spd_short_options = "dsal:c:S:p:P:C:vDh";
-
 void options_print_help(char *argv[]);
 void options_print_version(void);
 void options_parse(int argc, char *argv[]);
