@@ -21,9 +21,13 @@
  * $Id: output.c,v 1.38 2008-06-27 12:28:48 hanke Exp $
  */
 
-#include "output.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
-#include "fdsetconv.c"
+#include <fdsetconv.h>
+#include <spd_utils.h>
+#include "output.h"
 #include "parse.h"
 
 #ifdef TEMP_FAILURE_RETRY	/* GNU libc */
@@ -44,7 +48,7 @@ safe_write(int fd, const void *buf, size_t count) {
 #endif /* TEMP_FAILURE_RETRY */
 
 #if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
-/* Added by Willie Walker - strndup, and getline are gcc-isms
+/* Added by Willie Walker - strndup is a gcc-ism
  */
 char *strndup ( const char *s, size_t n)
 {
@@ -58,55 +62,13 @@ char *strndup ( const char *s, size_t n)
                 nAvail = n + 1;
         else
                 nAvail = strlen(s) + 1;
-        p = malloc ( nAvail );
+        p = g_malloc ( nAvail );
         memcpy ( p, s, nAvail );
         p[nAvail - 1] = '\0';
 
         return p;
 }
-
-#define BUFFER_LEN 256
-ssize_t getline (char **lineptr, size_t *n, FILE *f)
-{
-        char ch;
-        size_t m = 0;
-        ssize_t buf_len = 0;
-        char * buf = NULL;
-        char * p = NULL;
-
-	if (errno != 0) {
-	        errno = 0;
-	}
-        while ( (ch = getc(f)) !=EOF )
-        {
-                if (errno != 0)
-                        return -1;
-                if ( m++ >= buf_len )
-                {
-                        buf_len += BUFFER_LEN;
-                        buf = (char *) realloc(buf, buf_len + 1);
-                        if ( buf == NULL )
-                        {
-                                return -1;
-                        }
-                        p = buf + buf_len - BUFFER_LEN;
-                }
-                *p = ch;
-                p++;
-                if ( ch == '\n' )
-                        break;
-        }
-        if ( m == 0 )
-        {
-                return -1;
-        } else {
-                *p = '\0';
-                *lineptr = buf;
-                *n = m;
-                return m;
-        }
-}
-#endif /* !(defined(__GLIBC__) || defined(_GNU_SOURCE)) */
+#endif /* !(defined(__GLIBC__) && defined(_GNU_SOURCE)) */
 
 void
 output_set_speaking_monitor(TSpeechDMessage *msg, OutputModule *output)
@@ -121,10 +83,19 @@ OutputModule*
 get_output_module_by_name(char *name)
 {
   OutputModule *output;
-  output = g_hash_table_lookup(output_modules, name);
-  if (output == NULL || !output->working) output = NULL;
+  int i;
+
+  for (i = 0; i < g_list_length(output_modules); i++) {
+      output = g_list_nth_data(output_modules, i);
+      if(!strcmp(output->name, name)) {
+	  if(output->working)
+	      return output;
+	  else
+	      return NULL;
+      }
+  }
   
-  return output;
+  return NULL;
 }
 
 /* get_output_module tries to return a pointer to the
@@ -144,53 +115,49 @@ OutputModule*
 get_output_module(const TSpeechDMessage *message)
 {
   OutputModule *output = NULL;
-  GList *gl;
-  int i;
+  int i, len;
 
   if (message->settings.output_module != NULL){
     MSG(5, "Desired output module is %s", message->settings.output_module);
     output = get_output_module_by_name(message->settings.output_module);
-    if((output == NULL) || !output->working){
-      // If the requested module was not found or is not working,
-      // first try to use the default output module
-      if (GlobalFDSet.output_module != NULL){
-	MSG(3,"Warning: Didn't find prefered output module, using default");                
-	output = get_output_module_by_name(GlobalFDSet.output_module); 
-      }
-      if (output == NULL || !output->working){
-	MSG(3, "Couldn't load default output module, trying other modules");
-      }
+    if((output != NULL) && output->working) return output;
+  }
 
-      // Try all other output modules now to see if some of them
-      // is working
-      MSG(3, "Trying other output modules");
-      for (i=0; i<=g_list_length(output_modules_list)-1; i++){
-	gl = g_list_nth(output_modules_list, i);
-	if (!gl || !gl->data) break;
-	if (strcmp(gl->data, "dummy"))
-	  output = get_output_module_by_name(gl->data);
-	if ((output != NULL) && (output->working)){
-	  MSG(3, "Output module %s seems to be working, using it", gl->data);
-	  break;
-	}
-      }
+  MSG(3,"Warning: Didn't find preferred output module, using default");                
+  // If the requested module was not found or is not working,
+  // first try to use the default output module
+  if (GlobalFDSet.output_module != NULL)
+    output = get_output_module_by_name(GlobalFDSet.output_module); 
 
-      // Did we get something working? If not, use dummy (it will just play
-      // a pre-synthesized error message with some hints over and over).
-      if (output == NULL || !output->working){
-	MSG(1, "Error: No output module seems to be working, using the dummy output module");
-	output = get_output_module_by_name("dummy");
+  if (output != NULL && output->working) return output;
+
+  MSG(3, "Couldn't load default output module, trying other modules");
+
+  /* Try all other output modules other than dummy */
+  len = g_list_length(output_modules);
+  for (i = 0; i < len; i++) {
+      output = g_list_nth_data(output_modules, i);
+      if (0 == strcmp(output->name, "dummy"))
+           continue;
+
+      if (output->working) {
+          MSG(3, "Output module %s seems to be working, using it", output->name);
+          return output;
       }
+  }
+
+  // if we get here there are no good modules use the dummy
+  // a pre-synthesized error message with some hints over and over).
+  if (output == NULL || !output->working)
+    output = get_output_module_by_name("dummy");
 
       // Give up....
-      if (output == NULL){
-	MSG(1, "Error: No output module working, not even dummy, no sound produced!\n");
-	return NULL;
-      }
-    }
-  }
+  if (output == NULL)
+    MSG(1, "Error: No output module working, not even dummy, no sound produced!\n");
+
   return output;
 }
+
 void
 static output_lock(void)
 {
@@ -221,7 +188,7 @@ output_read_reply(OutputModule *output)
     /* Wait for activity on the socket, when there is some,
        read all the message line by line */
     do{
-	bytes = getline(&line, &N, output->stream_out);	
+	bytes = spd_getline(&line, &N, output->stream_out);
 	if (bytes == -1){
 	    MSG(2, "Error: Broken pipe to module.");
 	    output->working = 0;
@@ -236,7 +203,7 @@ output_read_reply(OutputModule *output)
     }while( !errors && !((strlen(line) < 4) || (line[3] == ' ')));
 
     if(line != NULL)
-	free(line);	/* getline allocates with malloc and realloc. */
+	g_free(line);
 
     if(errors) {
 	g_string_free(rstr, TRUE);
@@ -253,7 +220,7 @@ output_read_reply2(OutputModule *output)
     char *reply;
     
 
-    reply = malloc( 1024 * sizeof(char));
+    reply = g_malloc( 1024 * sizeof(char));
 
     bytes = read(output->pipe_out[0], reply, 1024);
     reply[bytes] = 0;
@@ -315,10 +282,10 @@ output_send_data(char* cmd, OutputModule *output, int wfr)
     return 0;
 }
 
-int
-_output_get_voices(OutputModule *module)
+static SPDVoice**
+output_get_voices(OutputModule *module)
 {
-  VoiceDescription** voice_dscr;
+  SPDVoice** voice_dscr;
   GString *reply;
   gchar **lines;
   gchar **atoms;
@@ -330,21 +297,20 @@ _output_get_voices(OutputModule *module)
 
   if (module == NULL){
     MSG(1, "ERROR: Can't list voices for broken output module");
-    OL_RET(-1);
+    OL_RET(NULL);
   }
   output_send_data("LIST VOICES\n", module, 0);
   reply = output_read_reply(module);
 
   if (reply == NULL){
       output_unlock();
-      voice_dscr = NULL;
-      return -1;
+      return NULL;
   }
 
   //TODO: only 256 voices supported here
   lines = g_strsplit(reply->str, "\n", 256);
   g_string_free(reply, TRUE);
-  voice_dscr = malloc(256*sizeof(VoiceDescription*));
+  voice_dscr = g_malloc(256*sizeof(SPDVoice*));
   for (i = 0; !errors && (lines[i] != NULL); i++) {
     MSG(1, "LINE here:|%s|", lines[i]);
     if (strlen(lines[i])<=4){
@@ -362,10 +328,10 @@ _output_get_voices(OutputModule *module)
       errors = TRUE;
       } else {
         //Fill in VoiceDescription
-        voice_dscr[i] = (VoiceDescription*) malloc(sizeof(VoiceDescription));
-        voice_dscr[i]->name=strdup(atoms[0]);
-        voice_dscr[i]->language=strdup(atoms[1]);
-        voice_dscr[i]->dialect=strdup(atoms[2]);
+        voice_dscr[i] = g_malloc(sizeof(SPDVoice));
+        voice_dscr[i]->name=g_strdup(atoms[0]);
+        voice_dscr[i]->language=g_strdup(atoms[1]);
+        voice_dscr[i]->variant=g_strdup(atoms[2]);
       }
     if (atoms != NULL)
       g_strfreev(atoms);
@@ -376,13 +342,11 @@ _output_get_voices(OutputModule *module)
   voice_dscr[i] = NULL;
   g_strfreev(lines);
 
-  module->voices=voice_dscr;
-
   output_unlock();
-  return ret;
+  return voice_dscr;
 }
 
-VoiceDescription**
+SPDVoice**
 output_list_voices(char* module_name)
 {
   OutputModule *module;
@@ -392,7 +356,7 @@ output_list_voices(char* module_name)
     MSG(1, "ERROR: Can't list voices for module %s", module_name);
     return NULL;
   }
-  return module->voices;
+  return output_get_voices(module);
 }
 
 #define SEND_CMD_N(cmd) \
@@ -424,13 +388,11 @@ output_list_voices(char* module_name)
        g_string_append_printf(set_str, #name"=NULL\n"); \
     }
 #define ADD_SET_STR_C(name, fconv) \
-    val = fconv(msg->settings.name); \
+    val = fconv(msg->settings.msg_settings.name); \
     if (val != NULL){ \
        g_string_append_printf(set_str, #name"=%s\n", val); \
-    }else{ \
-       g_string_append_printf(set_str, #name"=NULL\n"); \
     } \
-    spd_free(val);
+    g_free(val);
 
 int
 output_send_settings(TSpeechDMessage *msg, OutputModule *output)
@@ -441,15 +403,32 @@ output_send_settings(TSpeechDMessage *msg, OutputModule *output)
 
     MSG(4, "Module set parameters.");
     set_str = g_string_new("");
-    ADD_SET_INT(pitch);
-    ADD_SET_INT(rate);
-    ADD_SET_INT(volume);
+    g_string_append_printf(set_str, "pitch=%d\n",
+                           msg->settings.msg_settings.pitch);
+    g_string_append_printf(set_str, "rate=%d\n",
+                           msg->settings.msg_settings.rate);
+    g_string_append_printf(set_str, "volume=%d\n",
+                           msg->settings.msg_settings.volume);
     ADD_SET_STR_C(punctuation_mode, EPunctMode2str);
     ADD_SET_STR_C(spelling_mode, ESpellMode2str);
     ADD_SET_STR_C(cap_let_recogn, ECapLetRecogn2str);
-    ADD_SET_STR(language);
-    ADD_SET_STR_C(voice, EVoice2str);
-    ADD_SET_STR(synthesis_voice)
+    val = EVoice2str(msg->settings.msg_settings.voice_type);
+    if (val != NULL){
+        g_string_append_printf(set_str, "voice=%s\n", val);
+    }
+    g_free(val);
+    if (msg->settings.msg_settings.voice.language != NULL) {
+        g_string_append_printf(set_str, "language=%s\n",
+                               msg->settings.msg_settings.voice.language);
+    } else {
+       g_string_append_printf(set_str, "language=NULL\n");
+    }
+    if (msg->settings.msg_settings.voice.name != NULL) {
+       g_string_append_printf(set_str, "synthesis_voice=%s\n",
+                              msg->settings.msg_settings.voice.name);
+    } else {
+        g_string_append_printf(set_str, "synthesis_voice=NULL\n");
+    }
 
     SEND_CMD_N("SET");
     SEND_DATA_N(set_str->str);
@@ -529,7 +508,7 @@ output_send_debug(OutputModule *output, int flag, char* log_path)
     if (flag){
       cmd_str = g_strdup_printf("DEBUG ON %s \n", log_path);
       err = output_send_data(cmd_str, output, 1);
-      spd_free(cmd_str);
+      g_free(cmd_str);
       if (err){
 	MSG(3, "ERROR: Can't set debugging on for output module %s", output->name);
 	OL_RET(-1);
@@ -578,10 +557,10 @@ output_speak(TSpeechDMessage *msg)
 
     switch(msg->settings.type)
         {
-        case MSGTYPE_TEXT: SEND_CMD("SPEAK") break;
-        case MSGTYPE_SOUND_ICON: SEND_CMD("SOUND_ICON"); break;
-        case MSGTYPE_CHAR: SEND_CMD("CHAR"); break;
-        case MSGTYPE_KEY: SEND_CMD("KEY"); break;
+        case SPD_MSGTYPE_TEXT: SEND_CMD("SPEAK") break;
+        case SPD_MSGTYPE_SOUND_ICON: SEND_CMD("SOUND_ICON"); break;
+        case SPD_MSGTYPE_CHAR: SEND_CMD("CHAR"); break;
+        case SPD_MSGTYPE_KEY: SEND_CMD("KEY"); break;
         default: MSG(2,"Invalid message type in output_speak()!");
         }
 
@@ -686,13 +665,13 @@ output_module_is_speaking(OutputModule *output, char **index_mark)
 	    retcode = 0;
 	    MSG2(5, "output_module", "Received event:\n %s", response->str);
 	    if (!strncmp(response->str, "701", 3))
-		*index_mark = (char*) strdup("__spd_begin");
+		*index_mark = (char*) g_strdup("__spd_begin");
 	    else if (!strncmp(response->str, "702", 3))
-		*index_mark = (char*) strdup("__spd_end");
+		*index_mark = (char*) g_strdup("__spd_end");
 	    else if (!strncmp(response->str, "703", 3))
-		*index_mark = (char*) strdup("__spd_stopped");
+		*index_mark = (char*) g_strdup("__spd_stopped");
 	    else if (!strncmp(response->str, "704", 3))
-		*index_mark = (char*) strdup("__spd_paused");
+		*index_mark = (char*) g_strdup("__spd_paused");
 	    else if (!strncmp(response->str, "700", 3)) {
 		char *p;
 		p = strchr(response->str, '\n');
@@ -774,13 +753,13 @@ output_close(OutputModule *module)
     MSG(4, "Waiting for module pid %d", module->pid);
     ret = waitpid_with_timeout(module->pid, NULL, 0, 1000); 
     if (ret > 0){
-	MSG(3, "Ok, closed succesfully.");
+	MSG(4, "Ok, module closed successfully.");
     }else if (ret == 0){
 	int ret2;
-	MSG(1, "ERROR: Timed out when waiting for child cancelation");
+	MSG(1, "ERROR: Timed out when waiting for child cancellation");
 	MSG(3, "Killing the module");
 	kill(module->pid, SIGKILL);
-	MSG(3, "Waiting until the child terminates.");
+	MSG(4, "Waiting until the child terminates.");
 	ret2 = waitpid_with_timeout(module->pid, NULL, 0, 1000);
 	if (ret2 > 0){
 	    MSG(3, "Module terminated");
@@ -806,7 +785,7 @@ output_check_module(OutputModule* output)
 
     if(output == NULL) return -1;
 
-    MSG(3, "Output module working status: %d (pid:%d)", output->working, output->pid);
+    MSG(4, "Output module working status: %d (pid:%d)", output->working, output->pid);
 
     if (output->working == 0){
         /* Investigate on why it crashed */
@@ -850,7 +829,7 @@ escape_dot(char *otext)
 
     if (otext == NULL) return NULL;
 
-    MSG2(5, "escaping", "Incomming text: |%s|", otext);
+    MSG2(5, "escaping", "Incoming text: |%s|", otext);
 
     ootext = otext;
 
@@ -896,7 +875,7 @@ escape_dot(char *otext)
         ret = otext;
     }else{
         g_string_append(ntext, otext);
-        free(ootext);
+        g_free(ootext);
         ret = ntext->str;
 	g_string_free(ntext, 0);
     }
